@@ -4,7 +4,7 @@ use warnings;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '0.03';
+$VERSION = '0.05';
 
 #----------------------------------------------------------------------------
 
@@ -138,7 +138,7 @@ sub meta_spec_ok {
     }
 }
 
-q( Currently Listening To: KMDFM - "Hau Ruck" );
+q( Currently Listening To: Curve - "Gift" );
 
 __END__
 
@@ -176,7 +176,6 @@ for Miss Barbell Productions, L<http://www.missbarbell.co.uk>
 =head1 COPYRIGHT AND LICENSE
 
   Copyright (C) 2007 Barbie for Miss Barbell Productions
-  All Rights Reserved.
 
   This module is free software; you can redistribute it and/or 
   modify it under the same terms as Perl itself.
